from http import HTTPStatus
from typing import Dict

import requests


def _do_get_json(url, params) -> Dict:
    r = requests.get(url, params=params)
    if r.status_code != HTTPStatus.OK:
        raise Exception(f"Exception requesting JSON from {url} with params {params}")
    return r.json()



import collections


def dict_merge(dct, merge_dct):
    """ Recursive dict merge. Inspired by :meth:``dict.update()``, instead of
    updating only top-level keys, dict_merge recurses down into dicts nested
    to an arbitrary depth, updating keys. The ``merge_dct`` is merged into
    ``dct``.
    :param dct: dict onto which the merge is executed
    :param merge_dct: dct merged into dct
    :return: None
    """
    for k, v in merge_dct.items():
        if (k in dct and isinstance(dct[k], dict)
                and isinstance(merge_dct[k], collections.Mapping)):
            dict_merge(dct[k], merge_dct[k])
        else:
            dct[k] = merge_dct[k]



import logging
from inspect import Parameter, signature

from fxq.core.beans.factory.BeanFactory import BeanFactoryException
from fxq.core.context.ApplicationContext import application_context

LOGGER = logging.getLogger("Autowired")


def Autowired(name: str = None, type: type = None, *args, **kwargs):
    '''
    Decorator which can either be called directly or added to a method to inject beans dynamically.
    Should be primarily used to decorate a contructor whereby the required parameters will be
    injected at instantiation time.
    :param name: Name of the Bean as defined in the name
    :param type: Class of the Bean to be retreived
    :return: Returns the requested instance from the Bean factory
    '''
    if isinstance(name, str):
        return _AutowiredField(name, type)
    elif name is None and type is not None:
        return _AutowiredField(name, type)
    elif hasattr(name, '__name__') and name.__name__ == '__init__':
        return _AutowiredConstructor(name)
    else:
        LOGGER.error("Unsupported use of the Autowired decorator, call from the constructor or directly")
        raise Exception("Unsupported use of Autowired decorator")


def _AutowiredField(name: str, type: type):
    return application_context.get_bean(name, type)


def _AutowiredConstructor(constructor):
    LOGGER.debug("Attempting to Autowire constructor fields")

    injectable_beans = {}

    for parameter in signature(constructor).parameters.values():
        p: Parameter = parameter
        try:
            if parameter.name != 'self':
                injectable_beans[p.name] = _AutowiredField(name=p.name, type=None)
        except BeanFactoryException as e:
            LOGGER.warning("Unable to inject bean of name %s" % p.name)

    def injecting_constructor(instance, *args, **kwargs):
        constructor(instance, *args, **injectable_beans, **kwargs)

    return injecting_constructor



import yaml

from fxq.commons import dict_merge


class Environment:
    PROPERTY_FILE_PREFIX = 'app'

    def __init__(self, active_profiles=[], base_dir=""):
        self._base_dir = base_dir
        self.active_profiles = active_profiles
        self._properties = {}
        self._load_properties()

    def get_property(self, key):
        try:
            retval = self._properties
            for k in key.split("."):
                retval = retval.get(k)
            return retval
        except AttributeError:
            raise EnvironmentError(
                f"Property: {key} not found! Please define {key} in your {Environment.PROPERTY_FILE_PREFIX}.yml files, "
                f"currently active profiles are {self.active_profiles}"
            ) from None

    def _load_properties(self):
        # Load Initial no-profile file
        base_dir = f"{self._base_dir}/" if self._base_dir != "" else ""
        self._properties = yaml.load(open(f"{base_dir}{Environment.PROPERTY_FILE_PREFIX}.yml").read(),
                                     Loader=yaml.SafeLoader)

        # Load profile files if any active profiles are set
        for p in self.active_profiles:
            dict_merge(
                self._properties,
                yaml.load(open(f"{base_dir}{Environment.PROPERTY_FILE_PREFIX}.{p}.yml").read(), Loader=yaml.SafeLoader)
            )


class EnvironmentError(Exception):
    pass



import json
from typing import List, TypeVar, Type

import jsonpickle
from multipledispatch import dispatch

from fxq.commons._reflection_utils import _deserialize_json_object_to, _deserialize_json_list_to

T = TypeVar('T')


class DictMarshal:

    @dispatch(list)
    def to_dict(self, lst: list) -> List[dict]:
        """
        Serializes the given list of objects into a list of dicts
        :param lst: List of Objects to be serialized
        :return: List of dicts representing the objects
        """
        return [self.to_dict(i) for i in lst]

    @dispatch(object)
    def to_dict(self, obj: object) -> dict:
        """
        Serializes the given object into a dict
        :param obj: Object to be serialized
        :return: Dictionary representing the object
        """
        return json.loads(jsonpickle.encode(obj, unpicklable=False))

    @dispatch(dict, object)
    def from_dict(self, dict_: dict, class_: Type[T]) -> T:
        """
        Deserializes the dictionary into an object of the specified class

        Classes with complex field types must include a class level dictionary defining types
        i.e.
        ```python
        class Employee:
            types = {"emergencyContacts": List[EmergencyContact]}

            def __init__(self):
                self.id: int = None
                self.name: str = None
                self.hireDate: str = None
                self.current: bool = None
                self.emergencyContacts: List[EmergencyContact] = None
        ```

        :param dict_: Dictionary to be deserialized
        :param class_: Class to deserialize dictionary to
        :return: Returns an instance of class_
        """
        return _deserialize_json_object_to(dict_, class_)

    @dispatch(list, object)
    def from_dict(self, dict_list: List[dict], class_: Type[T]) -> List[T]:
        """
        Deserializes the dictionary list into an object list of the specified class

        Classes with complex field types must include a class level dictionary defining types
        i.e.
        ```python
        class Employee:
            types = {"emergencyContacts": List[EmergencyContact]}

            def __init__(self):
                self.id: int = None
                self.name: str = None
                self.hireDate: str = None
                self.current: bool = None
                self.emergencyContacts: List[EmergencyContact] = None
        ```

        :param dict_list: Dictionary to be deserialized
        :param class_: Class to deserialize dictionary to
        :return: Returns an list populated with instances of class_
        """
        return _deserialize_json_list_to(dict_list, class_)



import logging
from enum import Enum


class Impact(Enum):
    NONE = 0, "None"
    LOW = 1, "Low"
    MEDIUM = 2, "Medium"
    HIGH = 3, "High"

    @property
    def id(self) -> int:
        """ID for the Impact, i.e. 0 for None"""
        return self.value[0]

    @property
    def name(self) -> str:
        """Name of the Impact Enum"""
        return self.value[1]

    def get_id(self) -> int:
        """
        Get the ID of the Enum

        :return: Returns the ID of the Enum
        """
        return self.id

    def get_name(self) -> str:
        """
        Return the Name of the Enum

        :return: Returns the Name as a String
        """
        return self.name

    @staticmethod
    def value_of(value: str or int):
        """
        Parses the provided value into an Impact Enum

        i.e.

        0 -> Impact.NONE
        2 -> Impact.MEDIUM

        :param value: Value to parse, String or Id
        :return: Returns the Impact Enum
        """
        try:
            try:
                return [e for e in Impact.__members__.values() if e.id == value][0]
            except IndexError as ie:
                logging.debug(
                    f"Failed to match the id of '{value}'")

            return [e for e in Impact.__members__.values() if e.name.lower() == value.lower()][0]

        except IndexError as ie:
            raise Exception(
                f"Failed to match value '{value}' as an id or name") from None


class Type(Enum):
    CENTRAL_BANKS = 1, "Central Banks"
    CONFIDENCE_INDICES = 2, "Confidence Indices"
    CONSUMPTION_AND_INFLATION = 3, "Consumption & Inflation"
    EMPLOYMENT = 4, "Employment"
    GOVERNMENT = 5, "Government"
    HOLIDAYS = 6, "Holidays"
    INDUSTRIAL_AND_NON_INDUSTRIAL_ACTIVITY = 7, "Industrial & Non-Industrial Activity"
    INTEREST_RATES = 9, "Interest Rates"
    LIQUIDITY_AND_BALANCE = 10, "Liquidity & Balance"

    @property
    def id(self) -> int:
        """ID for the Type, i.e. 1 for Central Banks"""
        return self.value[0]

    @property
    def name(self) -> str:
        """Name of the Type Enum"""
        return self.value[1]

    def get_id(self) -> int:
        """
        Get the ID of the Enum

        :return: Returns the ID of the Enum
        """
        return self.id

    def get_name(self) -> str:
        """
        Return the Name of the Enum

        :return: Returns the Name as a String
        """
        return self.name

    @staticmethod
    def value_of(value: str or int):
        """
        Parses the provided value into an Type Enum

        :param value: Value to parse, String or Id
        :return: Returns the Type Enum
        """
        try:
            try:
                return [e for e in Type.__members__.values() if e.id == value][0]
            except IndexError as ie:
                logging.debug(
                    f"Failed to match the id of '{value}'")

            return [e for e in Type.__members__.values() if e.name.lower() == value.lower()][0]

        except IndexError as ie:
            raise Exception(
                f"Failed to match value '{value}' as an id or name") from None



import logging

from fxq.core.beans.factory import BeanFactory


class ApplicationContext(BeanFactory):
    LOGGER = logging.getLogger("ApplicationContext")

    def __init__(self):
        super(ApplicationContext, self).__init__()
        ApplicationContext.LOGGER.info("Application Context Initialized")


application_context = ApplicationContext()



import pathlib

from setuptools import setup

setup(
    name='fxq-commons',
    version='1.1.1',
    packages=[
        'fxq.core',
        'fxq.core.beans',
        'fxq.core.beans.factory',
        'fxq.core.beans.factory.annotation',
        'fxq.core.context',
        'fxq.core.stereotype',
        'fxq.commons',
        'fxq.env',
        'fxq.model'
    ],
    url='https://bitbucket.org/fxqlabs-oss/fxq-commons/',
    license='MIT',
    author='Jonathan Turnock',
    author_email='jonathan.turnock@outlook.com',
    description='Common python utilities and scripts for fxq labs',
    long_description=(pathlib.Path(__file__).parent / "README.md").read_text(),
    long_description_content_type="text/markdown",
    install_requires=[
        'PyYAML',
        'stringcase',
        'requests',
        'multidispatch',
        'jsonpickle'
    ]
)



from enum import Enum


class Country(Enum):
    AFGHANISTAN = "AF", "AFG",
    ALBANIA = "AL", "ALB",
    ALGERIA = "DZ", "DZA",
    AMERICAN_SAMOA = "AS", "ASM",
    ANDORRA = "AD", "AND",
    ANGOLA = "AO", "AGO",
    ANGUILLA = "AI", "AIA",
    ANTARCTICA = "AQ", "ATA",
    ANTIGUA_AND_BARBUDA = "AG", "ATG",
    ARGENTINA = "AR", "ARG",
    ARMENIA = "AM", "ARM",
    ARUBA = "AW", "ABW",
    AUSTRALIA = "AU", "AUS",
    AUSTRIA = "AT", "AUT",
    AZERBAIJAN = "AZ", "AZE",
    BAHAMAS = "BS", "BHS",
    BAHRAIN = "BH", "BHR",
    BANGLADESH = "BD", "BGD",
    BARBADOS = "BB", "BRB",
    BELARUS = "BY", "BLR",
    BELGIUM = "BE", "BEL",
    BELIZE = "BZ", "BLZ",
    BENIN = "BJ", "BEN",
    BERMUDA = "BM", "BMU",
    BHUTAN = "BT", "BTN",
    BOLIVIA = "BO", "BOL",
    BONAIRE_SINT_EUSTATIUS_AND_SABA = "BQ", "BES",
    BOSNIA_AND_HERZEGOVINA = "BA", "BIH",
    BOTSWANA = "BW", "BWA",
    BOUVET_ISLAND = "BV", "BVT",
    BRAZIL = "BR", "BRA",
    BRITISH_INDIAN_OCEAN_TERRITORY = "IO", "IOT",
    BRUNEI_DARUSSALAM = "BN", "BRN",
    BULGARIA = "BG", "BGR",
    BURKINA_FASO = "BF", "BFA",
    BURUNDI = "BI", "BDI",
    CABO_VERDE = "CV", "CPV",
    CAMBODIA = "KH", "KHM",
    CAMEROON = "CM", "CMR",
    CANADA = "CA", "CAN",
    CAYMAN_ISLANDS = "KY", "CYM",
    CENTRAL_AFRICAN_REPUBLIC = "CF", "CAF",
    CHAD = "TD", "TCD",
    CHILE = "CL", "CHL",
    CHINA = "CN", "CHN",
    CHRISTMAS_ISLAND = "CX", "CXR",
    COCOS_ISLANDS = "CC", "CCK",
    COLOMBIA = "CO", "COL",
    COMOROS = "KM", "COM",
    CONGO_DR = "CD", "COD",
    CONGO = "CG", "COG",
    COOK_ISLANDS = "CK", "COK",
    COSTA_RICA = "CR", "CRI",
    CROATIA = "HR", "HRV",
    CUBA = "CU", "CUB",
    CURACAO = "CW", "CUW",
    CYPRUS = "CY", "CYP",
    CZECHIA = "CZ", "CZE",
    COTE_D_IVOIRE = "CI", "CIV",
    DENMARK = "DK", "DNK",
    DJIBOUTI = "DJ", "DJI",
    DOMINICA = "DM", "DMA",
    DOMINICAN_REPUBLIC = "DO", "DOM",
    ECUADOR = "EC", "ECU",
    EGYPT = "EG", "EGY",
    EL_SALVADOR = "SV", "SLV",
    EQUATORIAL_GUINEA = "GQ", "GNQ",
    ERITREA = "ER", "ERI",
    ESTONIA = "EE", "EST",
    ESWATINI = "SZ", "SWZ",
    ETHIOPIA = "ET", "ETH",
    FALKLAND_ISLANDS = "FK", "FLK",
    FAROE_ISLANDS = "FO", "FRO",
    FIJI = "FJ", "FJI",
    FINLAND = "FI", "FIN",
    FRANCE = "FR", "FRA",
    FRENCH_GUIANA = "GF", "GUF",
    FRENCH_POLYNESIA = "PF", "PYF",
    FRENCH_SOUTHERN_TERRITORIES = "TF", "ATF",
    GABON = "GA", "GAB",
    GAMBIA = "GM", "GMB",
    GEORGIA = "GE", "GEO",
    GERMANY = "DE", "DEU",
    GHANA = "GH", "GHA",
    GIBRALTAR = "GI", "GIB",
    GREECE = "GR", "GRC",
    GREENLAND = "GL", "GRL",
    GRENADA = "GD", "GRD",
    GUADELOUPE = "GP", "GLP",
    GUAM = "GU", "GUM",
    GUATEMALA = "GT", "GTM",
    GUERNSEY = "GG", "GGY",
    GUINEA = "GN", "GIN",
    GUINEA_BISSAU = "GW", "GNB",
    GUYANA = "GY", "GUY",
    HAITI = "HT", "HTI",
    HEARD_ISLAND_AND_MC_DONALD_ISLANDS = "HM", "HMD",
    HOLY_SEE = "VA", "VAT",
    HONDURAS = "HN", "HND",
    HONG_KONG = "HK", "HKG",
    HUNGARY = "HU", "HUN",
    ICELAND = "IS", "ISL",
    INDIA = "IN", "IND",
    INDONESIA = "ID", "IDN",
    IRAN = "IR", "IRN",
    IRAQ = "IQ", "IRQ",
    IRELAND = "IE", "IRL",
    ISLE_OF_MAN = "IM", "IMN",
    ISRAEL = "IL", "ISR",
    ITALY = "IT", "ITA",
    JAMAICA = "JM", "JAM",
    JAPAN = "JP", "JPN",
    JERSEY = "JE", "JEY",
    JORDAN = "JO", "JOR",
    KAZAKHSTAN = "KZ", "KAZ",
    KENYA = "KE", "KEN",
    KIRIBATI = "KI", "KIR",
    KOREA_NORTH = "KP", "PRK",
    KOREA_SOUTH = "KR", "KOR",
    KUWAIT = "KW", "KWT",
    KYRGYZSTAN = "KG", "KGZ",
    LAO_PEOPLE_S_DEMOCRATIC_REPUBLIC = "LA", "LAO",
    LATVIA = "LV", "LVA",
    LEBANON = "LB", "LBN",
    LESOTHO = "LS", "LSO",
    LIBERIA = "LR", "LBR",
    LIBYA = "LY", "LBY",
    LIECHTENSTEIN = "LI", "LIE",
    LITHUANIA = "LT", "LTU",
    LUXEMBOURG = "LU", "LUX",
    MACAO = "MO", "MAC",
    MADAGASCAR = "MG", "MDG",
    MALAWI = "MW", "MWI",
    MALAYSIA = "MY", "MYS",
    MALDIVES = "MV", "MDV",
    MALI = "ML", "MLI",
    MALTA = "MT", "MLT",
    MARSHALL_ISLANDS = "MH", "MHL",
    MARTINIQUE = "MQ", "MTQ",
    MAURITANIA = "MR", "MRT",
    MAURITIUS = "MU", "MUS",
    MAYOTTE = "YT", "MYT",
    MEXICO = "MX", "MEX",
    MICRONESIA = "FM", "FSM",
    MOLDOVA = "MD", "MDA",
    MONACO = "MC", "MCO",
    MONGOLIA = "MN", "MNG",
    MONTENEGRO = "ME", "MNE",
    MONTSERRAT = "MS", "MSR",
    MOROCCO = "MA", "MAR",
    MOZAMBIQUE = "MZ", "MOZ",
    MYANMAR = "MM", "MMR",
    NAMIBIA = "NA", "NAM",
    NAURU = "NR", "NRU",
    NEPAL = "NP", "NPL",
    NETHERLANDS = "NL", "NLD",
    NEW_CALEDONIA = "NC", "NCL",
    NEW_ZEALAND = "NZ", "NZL",
    NICARAGUA = "NI", "NIC",
    NIGER = "NE", "NER",
    NIGERIA = "NG", "NGA",
    NIUE = "NU", "NIU",
    NORFOLK_ISLAND = "NF", "NFK",
    NORTHERN_MARIANA_ISLANDS = "MP", "MNP",
    NORWAY = "NO", "NOR",
    OMAN = "OM", "OMN",
    PAKISTAN = "PK", "PAK",
    PALAU = "PW", "PLW",
    PALESTINE_STATE_OF = "PS", "PSE",
    PANAMA = "PA", "PAN",
    PAPUA_NEW_GUINEA = "PG", "PNG",
    PARAGUAY = "PY", "PRY",
    PERU = "PE", "PER",
    PHILIPPINES = "PH", "PHL",
    PITCAIRN = "PN", "PCN",
    POLAND = "PL", "POL",
    PORTUGAL = "PT", "PRT",
    PUERTO_RICO = "PR", "PRI",
    QATAR = "QA", "QAT",
    REPUBLIC_OF_NORTH_MACEDONIA = "MK", "MKD",
    ROMANIA = "RO", "ROU",
    RUSSIAN_FEDERATION = "RU", "RUS",
    RWANDA = "RW", "RWA",
    REUNION = "RE", "REU",
    SAINT_BARTHELEMY = "BL", "BLM",
    SAINT_HELENA_ASCENSION_AND_TRISTAN_DA_CUNHA = "SH", "SHN",
    SAINT_KITTS_AND_NEVIS = "KN", "KNA",
    SAINT_LUCIA = "LC", "LCA",
    SAINT_MARTIN = "MF", "MAF",
    SAINT_PIERRE_AND_MIQUELON = "PM", "SPM",
    SAINT_VINCENT_AND_THE_GRENADINES = "VC", "VCT",
    SAMOA = "WS", "WSM",
    SAN_MARINO = "SM", "SMR",
    SAO_TOME_AND_PRINCIPE = "ST", "STP",
    SAUDI_ARABIA = "SA", "SAU",
    SENEGAL = "SN", "SEN",
    SERBIA = "RS", "SRB",
    SEYCHELLES = "SC", "SYC",
    SIERRA_LEONE = "SL", "SLE",
    SINGAPORE = "SG", "SGP",
    SINT_MAARTEN = "SX", "SXM",
    SLOVAKIA = "SK", "SVK",
    SLOVENIA = "SI", "SVN",
    SOLOMON_ISLANDS = "SB", "SLB",
    SOMALIA = "SO", "SOM",
    SOUTH_AFRICA = "ZA", "ZAF",
    SOUTH_GEORGIA_AND_THE_SOUTH_SANDWICH_ISLANDS = "GS", "SGS",
    SOUTH_SUDAN = "SS", "SSD",
    SPAIN = "ES", "ESP",
    SRI_LANKA = "LK", "LKA",
    SUDAN = "SD", "SDN",
    SURINAME = "SR", "SUR",
    SVALBARD_AND_JAN_MAYEN = "SJ", "SJM",
    SWEDEN = "SE", "SWE",
    SWITZERLAND = "CH", "CHE",
    SYRIAN_ARAB_REPUBLIC = "SY", "SYR",
    TAIWAN = "TW", "TWN",
    TAJIKISTAN = "TJ", "TJK",
    TANZANIA_UNITED_REPUBLIC_OF = "TZ", "TZA",
    THAILAND = "TH", "THA",
    TIMOR_LESTE = "TL", "TLS",
    TOGO = "TG", "TGO",
    TOKELAU = "TK", "TKL",
    TONGA = "TO", "TON",
    TRINIDAD_AND_TOBAGO = "TT", "TTO",
    TUNISIA = "TN", "TUN",
    TURKEY = "TR", "TUR",
    TURKMENISTAN = "TM", "TKM",
    TURKS_AND_CAICOS_ISLANDS = "TC", "TCA",
    TUVALU = "TV", "TUV",
    UGANDA = "UG", "UGA",
    UKRAINE = "UA", "UKR",
    UNITED_ARAB_EMIRATES = "AE", "ARE",
    UNITED_KINGDOM = "GB", "GBR",
    UNITED_STATES_MINOR_OUTLYING_ISLANDS = "UM", "UMI",
    UNITED_STATES_OF_AMERICA = "US", "USA",
    URUGUAY = "UY", "URY",
    UZBEKISTAN = "UZ", "UZB",
    VANUATU = "VU", "VUT",
    VENEZUELA = "VE", "VEN",
    VIET_NAM = "VN", "VNM",
    VIRGIN_ISLANDS_BRIT = "VG", "VGB",
    VIRGIN_ISLANDS_US = "VI", "VIR",
    WALLIS_AND_FUTUNA = "WF", "WLF",
    WESTERN_SAHARA = "EH", "ESH",
    YEMEN = "YE", "YEM",
    ZAMBIA = "ZM", "ZMB",
    ZIMBABWE = "ZW", "ZWE",
    ALAND_ISLANDS = "AX", "ALA"

    @property
    def alpha2(self):
        """Alpha 2 Code of the Country"""
        return self.value[0]

    @property
    def alpha3(self):
        """Alpha 3 Code of the Country"""
        return self.value[1]

    def get_alpha2(self):
        """
        Get the Alpha2 Code of the Enum

        :return: Returns the alpha2 code as a string
        """
        return self.alpha2

    def get_alpha3(self):
        """
        Get the Alpha3 Code of the Enum

        :return: Returns the alpha3 code as a string
        """
        return self.alpha3

    @staticmethod
    def value_of(alpha_code: str):
        """
        Parses the provided alpha_code into the Enum, takes either the Alpha2 or Alpha3 code

        i.e.

        "AFG" -> FxqCountry.AFGHANISTAN

        :param alpha_code: Alpha2 or Alpha3 code for the country
        :return: Returns the FxqCountry Enum
        """
        if len(alpha_code) not in [2, 3]:
            raise Exception(
                f"Failed to find the value of '{alpha_code}' Unsupported Alpha Code Length")

        try:
            if len(alpha_code) == 2:
                return [e for e in Country.__members__.values() if e.alpha2 == alpha_code][0]
            elif len(alpha_code) == 3:
                return [e for e in Country.__members__.values() if e.alpha3 == alpha_code][0]
        except IndexError as ie:
            raise Exception(
                f"Failed to match the alpha code of '{alpha_code}'") from None



import logging
from enum import Enum


class Currency(Enum):
    AUSTRALIAN_DOLLAR = "AUD", "$",
    CANADIAN_DOLLAR = "CAD", "$",
    SWISS_FRANC = "CHF", "CHF",
    YUAN_RENMINBI = "CNY", "¥",
    EURO = "EUR", "€",
    POUND_STERLING = "GBP", "£",
    YEN = "JPY", "¥",
    MEXICAN_PESO = "MXN", "$"
    NORWEGIAN_KRONE = "NOK", "kr",
    NEW_ZEALAND_DOLLAR = "NZD", "$",
    SWEDISH_KRONA = "SEK", "kr",
    TURKISH_LIRA = "TRY", "TL",
    US_DOLLAR = "USD", "$",
    RAND = "ZAR", "R"

    @property
    def code(self):
        """Currency Code i.e. GBP"""
        return self.value[0]

    @property
    def symbol(self):
        """Symbol of the currency i.e £"""
        return self.value[1]

    def get_code(self):
        """
        Get the Code of the Currency

        :return: Returns the code as a string i.e. GBP
        """
        return self.code

    def get_symbol(self):
        """
        Get the Symbol of the Currency

        :return: Returns the symbol of the currency as a string
        """
        return self.symbol

    @staticmethod
    def value_of(code: str):
        """
        Parses the provided Code into the Currency enum

        i.e.

        "GBP" -> FxqCurrency.POUND_STERLING

        :param code: Currency Code, i.e. GBP
        :return: Returns the FxqCurrency Enum
        """
        try:
            try:
                return [e for e in Currency.__members__.values() if e.code == code][0]
            except IndexError as ie:
                logging.debug(
                    f"Failed to match the code of '{code}'")

            return [e for e in Currency.__members__.values() if e.symbol == code][0]

        except IndexError as ie:
            raise Exception(
                f"Failed to match '{code}' as a code or symbol") from None
